/* Provide hints for Tamarin's heuristics.
   In this case, we tell Tamarin to delay (make "Last", "L_") finding the
   sources for the LogDevice fact, since it doesn't provide useful information
   and can lead to non-termination. It is similar for LogCloud, but the trace
   is found faster if we don't use L_ for that. */
define(`LogCloud',`L_LogCloud($*)')
define(`LogDevice',`L_LogDevice($*)')

theory DECIM_protocol_correctness

/* Include DECIM model */
include(`DECIM-base.inc')
// Fix syntax mixup: ') 

// To find the basic trace faster, we first exclude the verification protocol
axiom no_checks:
  "not (Ex #i devid A f k.
     CheckedLog(devid, A, f, k) @ i )"

/* The protocol can run correctly. */

lemma protocol_correctness:
 exists-trace
 " /* It is possible that */
   Ex d R skR dkR m #i.
     /* R received an encrypted message m on device d */
     MsgReceived(d, R, skR, dkR, m) @ #i
     /* without the adversary compromising any device. */
   & not (Ex d2 A ltk dkR #j. 
             Compromise_Device(d2, A, ltk, dkR) @ #j)
 "

end

// vi:ft=spthy 
