/* Provide hints for Tamarin's heuristics.
   In this case, we tell Tamarin to delay (make "Last", "L_") finding the
   sources for the LogCloud and LogDevice facts, since they don't provide useful information
   and can lead to non-termination. */
//xdefine(`LogCloud',`L_LogCloud($*)')
define(`LogDevice',`L_LogDevice($*)')
// »
changequote`'changequote(`«',`»')dnl
define(«Device»,«F_Device($*)»)
define(«StoresKey»,«F_StoresKey($*)»)
define(«Cert»,«sign(<'cert',pk($2),$1>,$3)»)	// Cert(R,stk,ltk)
// Fix vim syntax coloring mixup: ') 

theory DECIM_detect

/* Include DECIM model */
include(«DECIM-base.inc»)

axiom only_one_device:
  " All devid1 devid2 #i #j.
    ( Start_Role('R',devid1) @ i &
      Start_Role('R',devid2) @ j )
    ==>
    ( #i = #j )
  "

axiom only_one_Lora:
  " All l1 l2 #i #j.
    ( Is_Type('LogMaintainer',l1) @ i &
      Is_Type('LogMaintainer',l2) @ j )
    ==>
    ( l1 = l2 )
  "

axiom at_most_two_of_a_role:
  " All r p1 p2 p3 #i1 #i2 #i3 .
    ( Start_Role(r,p1) @ #i1 &
      Start_Role(r,p2) @ #i2 &
      Start_Role(r,p3) @ #i3 )
    ==>
    ( ( #i1 = #i2 ) | ( #i1 = #i3 ) | ( #i2 = #i3 ) )
  "

//// The following axiom helps finding the trace
//axiom only_one_of_each:
//  " All r devid1 devid2 #i #j.
//    ( Start_Role(r,devid1) @ #i &
//      Start_Role(r,devid2) @ #j )
//    ==>
//    ( #i = #j )
//  "

axiom limit_receives:
  " All devid a1 a2 b1 b2 c1 c2 d1 d2 #i1 #i2 .
    ( MsgReceived(devid, a1, b1, c1, d1) @ #i1 &
      MsgReceived(devid, a2, b2, c2, d2) @ #i2 )
    ==>
    ( #i1 = #i2 )
  "

// We want this to hold even if there is only one compromise
axiom one_compromise:
  "(All #i1 #i2 devid1 devid2 A1 A2 ltk1 ltk2 k1 k2.
     (
     Compromise_Device(devid1, A1, ltk1, k1) @ i1  &
     Compromise_Device(devid2, A2, ltk2, k2) @ i2 
     )
     ==>
     ( #i1 = #i2 )
   )"

// Similarly, we restrict to only one check
axiom one_check:
  "(All #i1 #i2 devid1 devid2 A1 A2 f1 f2 k1 k2.
     (
     CheckedLog(devid1, A1, f1, k1) @ i1  &
     CheckedLog(devid2, A2, f2, k2) @ i2 
     )
     ==>
     ( #i1 = #i2 )
   )"

/* Sanity check: can we actually reach the situation we aim for in the
 * 'detect' check? */
lemma detect_usage_trace_flag2:
  exists-trace
  " Ex devid ltkR stk m #i1 #i3 #i4 flag R k.
        ( /* If an honest paty sent to R an encrypted message m */
          ( MsgSent(R, ltkR, pk(stk), m) @ #i1 &
            // /* and R receives it */
            // MsgReceived(devid, R, ltkR, stk, m) @ #i2 &
            /* and the adversary knows m */
            K(m) @ #i3 &
            /* and this stk was not specifically compromised */
            /* in other words, the compromise was in a different epoch
             */
            not (Ex #j ltk . Compromise_Device(devid, R, ltk, stk) @ j ) &
            /* and we afterwards check the log */
            CheckedLog(devid, R, flag, k ) @ #i4 & 
            #i1 < #i4
          )
        )
       "

/* Basic secrecy property, proven elsewhere */
axiom message_secrecy2:
  " All R ltkR stpk m #i.
        ( /* If S sent a message m to R */
          ( MsgSent(R, ltkR, stpk, m) @ i &
            /* without the adversary having compromised a device of R at
            some point */
            not (Ex #r devid stk. Compromise_Device(devid, R, ltkR, stk) @ r)
          )
          ==>
          ( /* then the adversary does not know it */
            not ( Ex #j. K(m) @ j)
          )
        )
       "


/* Main detection of usage property.
 */
lemma detect_usage_S_sends:
 "All d skR dkR m #i1 #i2 #i3 detectionresult R k.
      /* If S sent to R an encrypted message m,
         where pk(dkR)=ekR */
      ( MsgSent(R, skR, pk(dkR), m) @ #i1 &
        /* and the adversary knows m */
        K(m) @ #i2 &
        /* and the ephemeral key used by the sender
           was not compromised, i.e., the compromise 
           occurred in a different epoch
         */
        not (Ex #j sk . 
                Compromise_Device(d, R, sk, dkR) @ #j ) &
        /* and Robert afterwards checks the log */
        CheckedLog(d, R, detectionresult, k ) @ #i3 & 
        #i1 < #i3
      )
      ==>
      ( /* then we detect a compromise */
        (detectionresult = 'bad')
      )
 "

/* Main detection of usage property.
 */
lemma detect_usage_R_receives:
 "All d skR dkR dkR2 m #i1 #i2 #i3 #i4 detectionresult R k.
      /* If S sent to R an encrypted message m,
         where pk(dkR)=ekR */
      ( MsgSent(R, skR, pk(dkR), m) @ #i1 &
        // /* and R receives it */
        MsgReceived(d, R, skR, dkR2, m) @ #i2 &
        /* and the adversary knows m */
        K(m) @ #i3 &
        /* and the ephemeral key used by the sender was 
           not compromised, i.e., the compromise was in 
           a different epoch then when m was sent.
         */
        not (Ex #j sk . 
                Compromise_Device(d, R, sk, dkR) @ #j ) &
        /* and Robert afterwards checks the log */
        CheckedLog(d, R, detectionresult, k ) @ #i4 & 
        #i2 < #i4
      )
      ==>
      ( /* then we can detect a compromise */
        (detectionresult = 'bad')
      )
 "

end

// vi:ft=spthy 
