/* Provide hints for Tamarin's heuristics.
   In this case, we tell Tamarin to delay (make "Last", "L_") finding the
   sources for the LogCloud and LogDevice facts, since they don't provide useful information
   and can lead to non-termination. */
define(`LogCloud',`L_LogCloud($*)')
define(`LogDevice',`L_LogDevice($*)')
// »
changequote`'changequote(`«',`»')dnl
define(«DeviceStatus»,«F_DeviceStatus($*)»)
define(«Cert»,«sign(<'cert',pk($2),$1>,$3)»)	// Cert(R,stk,ltk)
// Fix vim syntax coloring mixup: ') 

theory DECIM_log_bad_trace

/* Include DECIM model */
include(«DECIM-base.inc»)

// To find the basic trace faster, we only have one instance of each
// role.
axiom one_role_instance:
  "(All #i #j role param1 param2 .
     (
     Start_Role(role,param1) @ i &
     Start_Role(role,param2) @ j
     )
     ==>
     ( #i = #j )
   )"

// Similarly, we restrict to only one log check
axiom one_check:
  "(All #i1 #i2 devid1 devid2 A1 A2 f1 f2 k1 k2.
     (
     CheckedLog(devid1, A1, f1, k1) @ i1  &
     CheckedLog(devid2, A2, f2, k2) @ i2 
     )
     ==>
     ( #i1 = #i2 )
   )"

axiom one_compromise:
 "(All #i1 #i2 devid1 devid2 R1 R2 ltkR1 ltkR2 stk1 stk2.
     ( Compromise_Device(devid1, R1, ltkR1, stk1) @ #i1 &
       Compromise_Device(devid2, R2, ltkR2, stk2) @ #i2 )
     ==>
     ( ( #i1 = #i2 ) & ( stk1 = 'none') )
  )
 "

/* Sanity check on verification: detection may raise an alert
 */
lemma log_bad_trace2:
 exists-trace
 " Ex devid ltkR stpk m #i1 #i2 #i3 #i4 R k stkalt.
      ( /* If S sent to R an encrypted message m */
        MsgSent(R, ltkR, stpk, m) @ #i1 &
        /* and the adversary knows m */
        K(m) @ #i2 &
        /* compromise the long-term key, not the specific ephemeral
           key */
        Compromise_Device(devid, R, ltkR, stkalt) @ #i3  &
        not (pk(stkalt) = stpk) &
        /* and we check the log and find a compromise */
        CheckedLog(devid, R, 'bad', k ) @ #i4 
      )
 "

end

// vi:ft=spthy 
