/* Provide hints for Tamarin's heuristics.
   In this case, we tell Tamarin to delay (make "Last", "L_") finding the
   sources for the LogCloud and LogDevice facts, since they don't provide useful information
   and can lead to non-termination. */
define(`LogCloud',`L_LogCloud($*)')
define(`LogDevice',`L_LogDevice($*)')
// »
changequote`'changequote(`«',`»')dnl
define(«DeviceStatus»,«F_DeviceStatus($*)»)
define(«Cert»,«sign(<'cert',pk($2),$1>,$3)»)	// Cert(R,stk,ltk)
// Fix vim syntax coloring mixup: ') 

theory DECIM_log_good_trace

/* Include DECIM model */
include(«DECIM-base.inc»)

// To find the basic trace faster, we only have one instance of each
// role.
axiom one_role_instance:
  "(All #i #j role param1 param2 .
     (
     Start_Role(role,param1) @ i &
     Start_Role(role,param2) @ j
     )
     ==>
     ( #i = #j )
   )"

// Similarly, we restrict to only one log check
axiom one_check:
  "(All #i1 #i2 devid1 devid2 A1 A2 f1 f2 k1 k2.
     (
     CheckedLog(devid1, A1, f1, k1) @ i1  &
     CheckedLog(devid2, A2, f2, k2) @ i2 
     )
     ==>
     ( #i1 = #i2 )
   )"

/* Sanity check on verification: can finish trace with good log
 */
lemma log_good_trace2:
 exists-trace
 "Ex devid ltkR stk m #i1 #i2 R.
      ( /* If S sent to R an encrypted message m */
        MsgReceived(devid, R, ltkR, stk, m) @ #i1 &
        /* and we check the log afterwards and find no compromise */
        ( #i1 < #i2 ) &
        CheckedLog(devid, R, 'good', 'null' ) @ #i2 &
        /* and the adversary did not compromise anything */
        not (Ex #j devid2 R2 ltkR2 stk2.
                Compromise_Device(devid2, R2, ltkR2, stk2) @ j 
            ) 
      )
 "

end

// vi:ft=spthy 
